// Very small modification to make it run off realtime input from the microphone

import ddf.minim.*;
import ddf.minim.analysis.*;
 
Minim minim;
AudioInput in;
FFT fft;

PImage dst;

ArrayList<Contour> contours;
ArrayList<Contour> polygons;

// Variables qui définissent les "zones" du spectre
// Par exemple, pour les basses, on prend seulement les premières 4% du spectre total
float specLow = 0.03; // 3%
float specMid = 0.125;  // 12.5%
float specHi = 0.20;   // 20%

// Il reste donc 64% du spectre possible qui ne sera pas utilisé. 
// Ces valeurs sont généralement trop hautes pour l'oreille humaine de toute facon.

// Valeurs de score pour chaque zone
float scoreLow = 0;
float scoreMid = 0;
float scoreHi = 0;

// Valeur précédentes, pour adoucir la reduction
float oldScoreLow = scoreLow;
float oldScoreMid = scoreMid;
float oldScoreHi = scoreHi;

// Valeur d'adoucissement
float scoreDecreaseRate = 25;

import gab.opencv.*;
import processing.video.*;

OpenCV opencv;
Capture video;
PImage src, colorFilteredImage;

int buffer_size = 4096;  // also sets FFT size (frequency resolution)
float sample_rate = 44100;
float[] myBuffer;

void setup() {
  
  //Charger la librairie minim
  minim = new Minim(this);
 
  //Charger la chanson
  in = minim.getLineIn(Minim.MONO,buffer_size,sample_rate);
  myBuffer = new float[in.bufferSize()];
  //Créer l'objet FFT pour analyser la chanson
  fft = new FFT(in.bufferSize(), in.sampleRate());
  // Tapered window important for log-domain display
  fft.window(FFT.HAMMING);
  
  String[] cameras = Capture.list();

  if (cameras.length == 0) {
    println("There are no cameras available for capture.");
    exit();
  } else {
    println("Available cameras:");
    for (int i = 0; i < cameras.length; i++) {
      print(i);
      print(" ");
      println(cameras[i]);
    }
  }
  //video = new Capture(this, 320, 240, cameras[0]);
  video = new Capture(this, 320, 240);
  video.start();
  
  opencv = new OpenCV(this, video.width, video.height);
  video.start();  
  size(320, 720, P3D);
  //fullScreen(P3D);
}

void draw() {
  
  fft.forward(in.mix);
  
  //Calcul des "scores" (puissance) pour trois catégories de son
  //D'abord, sauvgarder les anciennes valeurs
  oldScoreLow = scoreLow;
  oldScoreMid = scoreMid;
  oldScoreHi = scoreHi;
  
  //Réinitialiser les valeurs
  scoreLow = 0;
  scoreMid = 0;
  scoreHi = 0;
 
  //Calculer les nouveaux "scores"
  for(int i = 0; i < fft.specSize()*specLow; i++)
  {
    scoreLow += fft.getBand(i);
  }
  
  for(int i = (int)(fft.specSize()*specLow); i < fft.specSize()*specMid; i++)
  {
    scoreMid += fft.getBand(i);
  }
  
  for(int i = (int)(fft.specSize()*specMid); i < fft.specSize()*specHi; i++)
  {
    scoreHi += fft.getBand(i);
  }
  
  background(0);
  // Read last captured frame
  if (video.available()) {
    video.read();
  }

  opencv.gray();
  opencv.threshold(70);
  dst = opencv.getOutput();

  contours = opencv.findContours();

  int combined = int(constrain(map(scoreHi+scoreMid+scoreLow,10,300,10,80),10,80));

  int red = int(constrain(map(scoreLow,10,50,0,255),0,255));
  int green = int(constrain(map(scoreMid,10,50,0,255),0,255));
  int blue = int(constrain(map(scoreHi,10,50,0,255),0,255));

  opencv.loadImage(video);
  opencv.calculateOpticalFlow();
  opencv.findSobelEdges(1,0);
  PImage sobel = opencv.getSnapshot();
  opencv.threshold(combined);
  PImage thresh = opencv.getSnapshot();

  stroke(red,green,blue); // Change this to change the colour of the lines
  image(thresh, 0, 0);  
  opencv.drawOpticalFlow();

  translate(0,240);
  stroke(blue,red,green); // Change this to change the colour of the lines
  image(thresh, 0, 0);  
  opencv.drawOpticalFlow();
  
  translate(0,240);
  stroke(green,blue,red); // Change this to change the colour of the lines
  image(thresh, 0, 0);  
  opencv.drawOpticalFlow();    

  translate(0,-480);
  
  noFill();
  strokeWeight(3);
  
  for (Contour contour : contours) {
    stroke(red, green, 0);
    contour.draw();
    
    stroke(red, green, blue);
    beginShape();
    for (PVector point : contour.getPolygonApproximation().getPoints()) {
      vertex(point.x, point.y);
    }
    endShape();

  }
  translate(0,240);

  for (Contour contour : contours) {
    stroke(0,red, green, 0);
    contour.draw();
    
    stroke(red, 0, blue);
    beginShape();
    for (PVector point : contour.getPolygonApproximation().getPoints()) {
      vertex(point.x, point.y);
    }
    endShape();

  }
  translate(0,240);

    for (Contour contour : contours) {
    stroke(0, red, blue);
    contour.draw();
    
    stroke(0, green, blue);
    beginShape();
    for (PVector point : contour.getPolygonApproximation().getPoints()) {
      vertex(point.x, point.y);
    }
    endShape();

  }

  translate(0,-480);
  //stroke(255);
  //strokeWeight(2);
  //line(video.width/2, video.height/2, video.width/2 + aveFlow.x*flowScale, video.height/2 + aveFlow.y*flowScale);
}
